@@ -1,15 +1,14 @@
 #include "mycrypt.h"
 #include <signal.h>
 
-/* decreased to 5 from 32 for Dropbear, this saves ~3kB of binary size */
-#define TAB_SIZE    5
+/* decreased to 4 from 32 for Dropbear, this saves ~3kB of binary size */
+#define TAB_SIZE    4
 
 struct _cipher_descriptor cipher_descriptor[TAB_SIZE] = {
 { NULL, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL },
 { NULL, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL },
 { NULL, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL },
 { NULL, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL },
-{ NULL, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL },
 };
 
 struct _hash_descriptor hash_descriptor[TAB_SIZE] = {
@@ -17,7 +16,6 @@ struct _hash_descriptor hash_descriptor[TAB_SIZE] = {
 { NULL, 0, 0, 0, NULL, NULL, NULL, NULL },
 { NULL, 0, 0, 0, NULL, NULL, NULL, NULL },
 { NULL, 0, 0, 0, NULL, NULL, NULL, NULL },
-{ NULL, 0, 0, 0, NULL, NULL, NULL, NULL },
 };
 
 struct _prng_descriptor prng_descriptor[TAB_SIZE] = {
@@ -25,7 +23,6 @@ struct _prng_descriptor prng_descriptor[TAB_SIZE] = {
 { NULL, NULL, NULL, NULL, NULL },
 { NULL, NULL, NULL, NULL, NULL },
 { NULL, NULL, NULL, NULL, NULL },
-{ NULL, NULL, NULL, NULL, NULL },
 };
 
 /* ch1-01-1 */
@@ -33,10 +30,10 @@ struct _prng_descriptor prng_descriptor[TAB_SIZE] = {
 void crypt_argchk(char *v, char *s, int d)
 {
 #ifdef SONY_PS2
- printf("_ARGCHK '%s' failure on line %d of file %s\n", 
+ printf("_ARGCHK '%s' failure on line %d of file %s\n",
          v, d, s);
-#else 
- fprintf(stderr, "_ARGCHK '%s' failure on line %d of file %s\n", 
+#else
+ fprintf(stderr, "_ARGCHK '%s' failure on line %d of file %s\n",
          v, d, s);
 #endif
  (void)raise(SIGABRT);
@@ -326,7 +323,7 @@ const char *crypt_build_settings =
    "   Twofish "
    #if defined(TWOFISH_SMALL) && defined(TWOFISH_TABLES)
        "(small, tables)\n"
-   #elif defined(TWOFISH_SMALL) 
+   #elif defined(TWOFISH_SMALL)
        "(small)\n"
    #elif defined(TWOFISH_TABLES)
        "(tables)\n"
@@ -343,6 +340,9 @@ const char *crypt_build_settings =
 #if defined(NOEKEON)
    "   Noekeon\n"
 #endif
+#if defined(SKIPJACK)
+   "   Skipjack\n"
+#endif
 
     "\nHashes built-in:\n"
 #if defined(SHA512)
@@ -354,6 +354,9 @@ const char *crypt_build_settings =
 #if defined(SHA256)
    "   SHA-256\n"
 #endif
+#if defined(SHA224)
+   "   SHA-224\n"
+#endif
 #if defined(TIGER)
    "   TIGER\n"
 #endif
@@ -369,6 +372,12 @@ const char *crypt_build_settings =
 #if defined(MD2)
    "   MD2\n"
 #endif
+#if defined(RIPEMD128)
+   "   RIPEMD128\n"
+#endif
+#if defined(RIPEMD160)
+   "   RIPEMD160\n"
+#endif
 
     "\nBlock Chaining Modes:\n"
 #if defined(CFB)
@@ -445,8 +454,8 @@ const char *crypt_build_settings =
 #if defined(TRY_UNRANDOM_FIRST)
     " TRY_UNRANDOM_FIRST "
 #endif
-#if defined(SMALL_PRIME_TAB)
-    " SMALL_PRIME_TAB "
+#if defined(LTC_TEST)
+    " LTC_TEST "
 #endif
     "\n"
 
